@@ -3,17 +3,19 @@ import sys, time, datetime, json
 
 import boto, boto.s3.key, boto.s3.bucket
 
-import aws
+import aws, _threadpool
 
 import logging
 log = logging.getLogger('motorboto.s3')
 
-FIVE_MB = 1024 * 1024 * 5
+FIVE_KB = 1024 * 5
+FIVE_MB = FIVE_KB * 1024
 FIVE_GB = FIVE_MB * 1024
 
 MAX_UPLOAD_CHUNK_SIZE = FIVE_MB
 
 from util import property_lazy
+from concurrent import futures
 
 class Connection(aws.Connection):
     """Top-level s3 connection, from which you can get buckets
@@ -357,7 +359,6 @@ class Key(object):
         if not async:
             return self._save(f, public, copy_from)
         else:
-            import _threadpool
             k = self._threadsafe_copy()
             
             a = _threadpool.executor.submit(k._save, f, public, copy_from)
@@ -409,12 +410,13 @@ class Key(object):
                     if not result.saved_from_copy:
                         if local_size(f) > MAX_UPLOAD_CHUNK_SIZE:
                             result.multipart = True
-                            log.info('DOING MULTIPART UPLOAD')
                             
+                            metadata['md5'] = local_checksum
                             mp = self.bucket.boto.initiate_multipart_upload(self.name,
                                                                             metadata=metadata)
+                                                                            
                             from helpful.fparts import file_parts
-                            fps = file_parts(f, MAX_UPLOAD_CHUNK_SIZE)
+                            fps = list(file_parts(f, MAX_UPLOAD_CHUNK_SIZE))
                             
                             parts = {
                                 _threadpool.executor.submit(mp.upload_part_from_file,
@@ -425,6 +427,7 @@ class Key(object):
                             for future in futures.as_completed(parts):
                                 index = parts[future]
                                 log.info('SAVED PART: %s (part #%s)' % (self.name, index))
+                                
                             mp.complete_upload()
                         else:
                             for k, v in self.metadata.items():
